@@ -94,18 +94,29 @@ void gravitate_client(Client *c, int sign) {
 }
 
 void select_client(Client *c) {
-#ifdef COLOURMAP
-	XInstallColormap(dpy, c->cmap);
+	if (current)
+		XSetWindowBorder(dpy, current->parent, current->screen->bg.pixel);
+	if (c) {
+		unsigned long bpixel;
+#ifdef VWM
+		if (is_sticky(c))
+			bpixel = c->screen->fc.pixel;
+		else
 #endif
-	client_update_current(current, c);
-	client_update_current(c, current);
-	XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
+			bpixel = c->screen->fg.pixel;
+		XSetWindowBorder(dpy, c->parent, bpixel);
+#ifdef COLOURMAP
+		XInstallColormap(dpy, c->cmap);
+#endif
+		XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
+	}
+	current = c;
 }
 
 #ifdef VWM
 void fix_client(Client *c) {
 	toggle_sticky(c);
-	client_update_current(c, current);
+	select_client(c);
 	update_net_wm_state(c);
 }
 #endif
@@ -130,8 +141,10 @@ void remove_client(Client *c) {
 	if (c->remove) {
 		LOG_DEBUG("\tremove_client() : setting WithdrawnState\n");
 		set_wm_state(c, WithdrawnState);
+#ifdef VWM
 		XDeleteProperty(dpy, c->window, xa_net_wm_desktop);
 		XDeleteProperty(dpy, c->window, xa_net_wm_state);
+#endif
 	}
 
 	ungravitate(c);
@@ -195,32 +208,18 @@ static int send_xmessage(Window w, Atom a, long x) {
 
 #ifdef SHAPE
 void set_shape(Client *c) {
+	int bounding_shaped;
 	int i, b;  unsigned int u;  /* dummies */
 
 	if (!have_shape) return;
 	/* Logic to decide if we have a shaped window cribbed from fvwm-2.5.10.
 	 * Previous method (more than one rectangle returned from
 	 * XShapeGetRectangles) worked _most_ of the time. */
-	XShapeSelectInput(dpy, c->window, ShapeNotifyMask);
-	if (XShapeQueryExtents(dpy, c->window, &(c->bounding_shaped), &i, &i,
-				&u, &u, &b, &i, &i, &u, &u) && c->bounding_shaped) {
-		LOG_DEBUG("%d shape extents\n", c->bounding_shaped);
+	if (XShapeQueryExtents(dpy, c->window, &bounding_shaped, &i, &i,
+				&u, &u, &b, &i, &i, &u, &u) && bounding_shaped) {
+		LOG_DEBUG("%d shape extents\n", bounding_shaped);
 		XShapeCombineShape(dpy, c->parent, ShapeBounding, 0, 0,
 				c->window, ShapeBounding, ShapeSet);
 	}
 }
 #endif
-
-void client_update_current(Client *c, Client *newcurrent) {
-	if (c) {
-		unsigned long bpixel;
-#ifdef VWM
-		if (is_sticky(c))
-			bpixel = c == newcurrent ? c->screen->fc.pixel : c->screen->bg.pixel;
-		else
-#endif
-			bpixel = c == newcurrent ? c->screen->fg.pixel : c->screen->bg.pixel;
-		XSetWindowBorder(dpy, c->parent, bpixel);
-	}
-	current = newcurrent;
-}
