@@ -21,15 +21,55 @@ static void current_to_head(void) {
 }
 
 static void handle_key_event(XKeyEvent *e) {
-	Client *c = find_client(e->window);
 	KeySym key = XKeycodeToKeysym(dpy,e->keycode,0);
+	Client *c;
+	int width_inc, height_inc;
+#ifdef VWM
+	ScreenInfo *current_screen = find_current_screen();
+#endif
 
-	if (!c)
-		c = current;
-	if (c) {
-		int width_inc = (c->width_inc > 1) ? c->width_inc : 16;
-		int height_inc = (c->height_inc > 1) ? c->height_inc : 16;
-		switch (key) {
+	switch(key) {
+		case KEY_NEW:
+			spawn(opt_term);
+			break;
+		case KEY_NEXT:
+			next();
+			if (XGrabKeyboard(dpy, e->root, False, GrabModeAsync, GrabModeAsync, CurrentTime) == GrabSuccess) {
+				XEvent ev;
+				do {
+					XMaskEvent(dpy, KeyPressMask|KeyReleaseMask, &ev);
+					if (ev.type == KeyPress && XKeycodeToKeysym(dpy,ev.xkey.keycode,0) == KEY_NEXT)
+						next();
+				} while (ev.type == KeyPress || XKeycodeToKeysym(dpy,ev.xkey.keycode,0) == KEY_NEXT);
+				XUngrabKeyboard(dpy, CurrentTime);
+			}
+			current_to_head();
+			break;
+#ifdef VWM
+		case XK_1: case XK_2: case XK_3: case XK_4:
+		case XK_5: case XK_6: case XK_7: case XK_8:
+			switch_vdesk(current_screen, KEY_TO_VDESK(key));
+			break;
+		case KEY_PREVDESK:
+			if (current_screen->vdesk > KEY_TO_VDESK(XK_1)) {
+				switch_vdesk(current_screen,
+						current_screen->vdesk - 1);
+			}
+			break;
+		case KEY_NEXTDESK:
+			if (current_screen->vdesk < KEY_TO_VDESK(XK_8)) {
+				switch_vdesk(current_screen,
+						current_screen->vdesk + 1);
+			}
+			break;
+#endif
+		default: break;
+	}
+	c = current;
+	if (c == NULL) return;
+	width_inc = (c->width_inc > 1) ? c->width_inc : 16;
+	height_inc = (c->height_inc > 1) ? c->height_inc : 16;
+	switch (key) {
 		case KEY_LEFT:
 			if (e->state & altmask) {
 				if ((c->width - width_inc) >= c->min_width)
@@ -102,39 +142,7 @@ static void handle_key_event(XKeyEvent *e) {
 			fix_client(c);
 			break;
 #endif
-		}
-	}
-	switch(key) {
-		case KEY_NEW:
-			spawn(opt_term);
-			break;
-		case KEY_NEXT:
-			next();
-			if (XGrabKeyboard(dpy, e->root, False, GrabModeAsync, GrabModeAsync, CurrentTime) == GrabSuccess) {
-				XEvent ev;
-				do {
-					XMaskEvent(dpy, KeyPressMask|KeyReleaseMask, &ev);
-					if (ev.type == KeyPress && XKeycodeToKeysym(dpy,ev.xkey.keycode,0) == KEY_NEXT)
-						next();
-				} while (ev.type == KeyPress || XKeycodeToKeysym(dpy,ev.xkey.keycode,0) == KEY_NEXT);
-				XUngrabKeyboard(dpy, CurrentTime);
-			}
-			current_to_head();
-			break;
-#ifdef VWM
-		case XK_1: case XK_2: case XK_3: case XK_4:
-		case XK_5: case XK_6: case XK_7: case XK_8:
-			switch_vdesk(KEY_TO_VDESK(key));
-			break;
-		case KEY_PREVDESK:
-			if (vdesk > KEY_TO_VDESK(XK_1))
-				switch_vdesk(vdesk - 1);
-			break;
-		case KEY_NEXTDESK:
-			if (vdesk < KEY_TO_VDESK(XK_8))
-				switch_vdesk(vdesk + 1);
-			break;
-#endif
+		default: break;
 	}
 	return;
 move_client:
@@ -157,6 +165,7 @@ static void handle_button_event(XButtonEvent *e) {
 				sweep(c); break;
 			case Button3:
 				XLowerWindow(dpy, c->parent); break;
+			default: break;
 		}
 	}
 }
@@ -165,6 +174,7 @@ static void handle_button_event(XButtonEvent *e) {
 static void handle_configure_request(XConfigureRequestEvent *e) {
 	Client *c = find_client(e->window);
 	XWindowChanges wc;
+	unsigned int value_mask = e->value_mask;
 
 	wc.sibling = e->above;
 	wc.stack_mode = e->detail;
@@ -172,10 +182,16 @@ static void handle_configure_request(XConfigureRequestEvent *e) {
 	wc.height = e->height;
 	if (c) {
 		ungravitate(c);
-		if (e->value_mask & CWWidth) c->width = e->width;
-		if (e->value_mask & CWHeight) c->height = e->height;
-		if (e->value_mask & CWX) c->x = e->x;
-		if (e->value_mask & CWY) c->y = e->y;
+		if (value_mask & CWWidth) c->width = e->width;
+		if (value_mask & CWHeight) c->height = e->height;
+		if (value_mask & CWX) c->x = e->x;
+		if (value_mask & CWY) c->y = e->y;
+		if (value_mask & CWStackMode && value_mask & CWSibling) {
+			Client *sibling = find_client(e->above);
+			if (sibling) {
+				wc.sibling = sibling->parent;
+			}
+		}
 		if (c->x == 0 && c->width >= DisplayWidth(dpy, c->screen->screen)) {
 			c->x -= c->border;
 		}
@@ -187,15 +203,19 @@ static void handle_configure_request(XConfigureRequestEvent *e) {
 		wc.x = c->x - c->border;
 		wc.y = c->y - c->border;
 		wc.border_width = c->border;
-		XConfigureWindow(dpy, c->parent, e->value_mask, &wc);
-		send_config(c);
+		LOG_XDEBUG("XConfigureWindow(dpy, parent(%x), %lx, &wc);\n", (unsigned int)c->parent, value_mask);
+		XConfigureWindow(dpy, c->parent, value_mask, &wc);
+		XMoveResizeWindow(dpy, c->window, 0, 0, c->width, c->height);
+		if ((value_mask & (CWX|CWY)) && !(value_mask & (CWWidth|CWHeight))) {
+			send_config(c);
+		}
 		wc.border_width = 0;
+	} else {
+		wc.x = c ? 0 : e->x;
+		wc.y = c ? 0 : e->y;
+		LOG_XDEBUG("XConfigureWindow(dpy, window(%x), %lx, &wc);\n", (unsigned int)e->window, value_mask);
+		XConfigureWindow(dpy, e->window, value_mask, &wc);
 	}
-
-	wc.x = c ? 0 : e->x;
-	wc.y = c ? 0 : e->y;
-	XConfigureWindow(dpy, e->window, e->value_mask, &wc);
-	LOG_DEBUG("handle_configure_request() : window configured to %dx%d+%d+%d\n", wc.width, wc.height, wc.x, wc.y);
 }
 
 static void handle_map_request(XMapRequestEvent *e) {
@@ -203,8 +223,8 @@ static void handle_map_request(XMapRequestEvent *e) {
 
 	if (c) {
 #ifdef VWM
-		if (c->vdesk != vdesk)
-			switch_vdesk(c->vdesk);
+		if (c->vdesk != c->screen->vdesk)
+			switch_vdesk(c->screen, c->vdesk);
 #endif
 		unhide(c, RAISE);
 	} else {
@@ -257,17 +277,23 @@ static void handle_property_change(XPropertyEvent *e) {
 static void handle_enter_event(XCrossingEvent *e) {
 	Client *c;
 
-	current_screen = find_screen(e->root);
 	if ((c = find_client(e->window))) {
 #ifdef VWM
-		if (c->vdesk != vdesk && !is_sticky(c))
+		if (c->vdesk != c->screen->vdesk)
 			return;
 #endif
 		select_client(c);
 		current_to_head();
-#ifdef MOUSE
-		grab_button(c->parent, grabmask2, AnyButton);
-#endif
+	}
+}
+
+static void handle_mappingnotify_event(XMappingEvent *e) {
+	XRefreshKeyboardMapping(e);
+	if (e->request == MappingKeyboard) {
+		int i;
+		for (i = 0; i < num_screens; i++) {
+			grab_keys_for_screen(&screens[i]);
+		}
 	}
 }
 
@@ -305,6 +331,8 @@ void event_main_loop(void) {
 				handle_property_change(&ev.xproperty); break;
 			case UnmapNotify:
 				handle_unmap_event(&ev.xunmap); break;
+			case MappingNotify:
+				handle_mappingnotify_event(&ev.xmapping); break;
 #ifdef SHAPE
 			default:
 				if (have_shape && ev.type == shape_event) {
