@@ -22,9 +22,6 @@ typedef struct {
 	unsigned long decorations;
 } PropMwmHints;
 
-#define MAXIMISE_HORZ        (1<<0)
-#define MAXIMISE_VERT        (1<<1)
-
 /* sanity on options */
 #if defined(INFOBANNER_MOVERESIZE) && !defined(INFOBANNER)
 # define INFOBANNER
@@ -52,6 +49,9 @@ typedef struct {
 #define RAISE           1
 #define NO_RAISE        0       /* for unhide() */
 
+#define MAXIMISE_HORZ        (1<<0)
+#define MAXIMISE_VERT        (1<<1)
+
 /* some coding shorthand */
 
 #define ChildMask       (SubstructureRedirectMask|SubstructureNotifyMask)
@@ -61,20 +61,25 @@ typedef struct {
 #define grab_pointer(w, mask, curs) \
 	(XGrabPointer(dpy, w, False, mask, GrabModeAsync, GrabModeAsync, \
 	None, curs, CurrentTime) == GrabSuccess)
-#define grab_button(w, mask, button) \
-	XGrabButton(dpy, button, (mask), w, False, ButtonMask, \
-	            GrabModeAsync, GrabModeSync, None, None); \
-	XGrabButton(dpy, button, LockMask|(mask), w, False, ButtonMask, \
-	            GrabModeAsync, GrabModeSync, None, None); \
-	if (numlockmask) { \
+#define grab_button(w, mask, button) do { \
+		XGrabButton(dpy, button, (mask), w, False, ButtonMask, \
+			    GrabModeAsync, GrabModeSync, None, None); \
+		XGrabButton(dpy, button, LockMask|(mask), w, False, ButtonMask,\
+			    GrabModeAsync, GrabModeSync, None, None); \
 		XGrabButton(dpy, button, numlockmask|(mask), w, False, \
-		            ButtonMask, GrabModeAsync, GrabModeSync, \
-		            None, None); \
-		XGrabButton(dpy, button, numlockmask|LockMask|(mask), w, False, \
-		            ButtonMask, GrabModeAsync, GrabModeSync, \
-		            None, None); \
-	}
+			    ButtonMask, GrabModeAsync, GrabModeSync, \
+			    None, None); \
+		XGrabButton(dpy, button, numlockmask|LockMask|(mask), w, False,\
+			    ButtonMask, GrabModeAsync, GrabModeSync, \
+			    None, None); \
+	} while (0)
 #define setmouse(w, x, y) XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y)
+#define get_mouse_position(xp,yp,root) do { \
+		Window dw; \
+		int di; \
+		unsigned int dui; \
+		XQueryPointer(dpy, root, &dw, &dw, xp, yp, &di, &di, &dui); \
+	} while (0)
 #define gravitate(c) gravitate_client(c, 1)
 #define ungravitate(c) gravitate_client(c, -1)
 
@@ -92,15 +97,14 @@ typedef struct {
 /* screen structure */
 
 typedef struct ScreenInfo ScreenInfo;
-
 struct ScreenInfo {
 	int screen;
 	Window root;
 	GC invert_gc;
-#ifdef VWM
-	XColor fg, bg, fc;
-#else
 	XColor fg, bg;
+#ifdef VWM
+	int vdesk;
+	XColor fc;
 #endif
 	char *display;
 };
@@ -108,13 +112,10 @@ struct ScreenInfo {
 /* client structure */
 
 typedef struct Client Client;
-
 struct Client {
-	Client  *next;
 	Window  window;
 	Window  parent;
 	ScreenInfo      *screen;
-
 #ifdef COLOURMAP
 	Colormap        cmap;
 #endif
@@ -133,11 +134,9 @@ struct Client {
 #ifdef VWM
 	int             vdesk;
 	int             sticky;
-#endif /* def VWM */
-#ifdef SHAPE
-	int             bounding_shaped;
 #endif
 	int             remove;  /* set when client needs to be removed */
+	Client  *next;
 };
 
 typedef struct Application Application;
@@ -154,29 +153,40 @@ struct Application {
 	Application *next;
 };
 
-/* declarations for global variables in main.c */
+/* Declarations for global variables in main.c */
 
-extern Display          *dpy;
-extern int              num_screens;
-extern ScreenInfo       *screens;
-extern ScreenInfo       *current_screen;
-extern Client           *current;
-extern volatile Window  initialising;
-extern XFontStruct      *font;
-extern Client           *head_client;
-extern Application      *head_app;
-extern Cursor           move_curs;
-extern Cursor           resize_curs;
-extern const char       *opt_display;
-extern const char       *opt_font;
-extern const char       *opt_fg;
-extern const char       *opt_bg;
+/* Commonly used X information */
+extern Display      *dpy;
+extern XFontStruct  *font;
+extern Cursor       move_curs;
+extern Cursor       resize_curs;
+extern int          num_screens;
+extern ScreenInfo   *screens;
+#ifdef SHAPE
+extern int          have_shape, shape_event;
+#endif
+
+/* Standard X protocol atoms */
+extern Atom xa_wm_state;
+extern Atom xa_wm_protos;
+extern Atom xa_wm_delete;
+extern Atom xa_wm_cmapwins;
+/* Motif atoms */
+extern Atom mwm_hints;
+/* EWMH atoms */
+#ifdef VWM
+extern Atom xa_net_wm_desktop;
+extern Atom xa_net_wm_state;
+extern Atom xa_net_wm_state_sticky;
+#endif
+
+/* Things that affect user interaction */
+extern unsigned int     numlockmask;
+extern unsigned int     grabmask1;
+extern unsigned int     grabmask2;
+extern unsigned int     altmask;
 extern const char       *opt_term[3];
 extern int              opt_bw;
-#ifdef VWM
-extern const char       *opt_fc;
-extern int              vdesk;
-#endif
 #ifdef SNAP
 extern int              opt_snap;
 #endif
@@ -185,29 +195,16 @@ extern int              solid_drag;
 #else
 # define solid_drag (0)
 #endif
-#ifdef SHAPE
-extern int              have_shape, shape_event;
-#endif
-extern unsigned int numlockmask;
-extern unsigned int grabmask2;
-extern unsigned int altmask;
+extern Application      *head_app;
 
-/* Standard X protocol atoms */
-extern Atom             xa_wm_state;
-extern Atom             xa_wm_protos;
-extern Atom             xa_wm_delete;
-extern Atom             xa_wm_cmapwins;
-/* Motif atoms */
-extern Atom             mwm_hints;
-/* EWMH atoms */
-extern Atom xa_net_wm_desktop;
-extern Atom xa_net_wm_state;
-extern Atom xa_net_wm_state_sticky;
+/* Client tracking information */
+extern Client           *head_client;
+extern Client           *current;
+extern volatile Window  initialising;
 
 /* client.c */
 
 Client *find_client(Window w);
-int wm_state(Client *c);
 void gravitate_client(Client *c, int sign);
 void select_client(Client *c);
 #ifdef VWM
@@ -218,17 +215,11 @@ void send_config(Client *c);
 void send_wm_delete(Client *c, int kill_client);
 void set_wm_state(Client *c, int state);
 void set_shape(Client *c);
-void client_update_current(Client *c, Client *newcurrent);
 
 /* events.c */
 
 void event_main_loop(void);
 
-/* main.c */
-
-int main(int argc, char *argv[]);
-void scan_windows(void);
-
 /* misc.c */
 
 extern int need_client_tidy;
@@ -236,7 +227,6 @@ extern int ignore_xerror;
 int handle_xerror(Display *dsply, XErrorEvent *e);
 void spawn(const char *const cmd[]);
 void handle_signal(int signo);
-void grab_keysym(Window w, unsigned int mask, KeySym keysym);
 
 /* new.c */
 
@@ -246,9 +236,7 @@ long get_wm_normal_hints(Client *c);
 /* screen.c */
 
 void drag(Client *c);
-void get_mouse_position(int *x, int *y, Window root);
 void moveresize(Client *c);
-void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2);
 void maximise_client(Client *c, int hv);
 void show_info(Client *c, KeySym key);
 void sweep(Client *c);
@@ -256,9 +244,11 @@ void unhide(Client *c, int raise_win);
 void next(void);
 #ifdef VWM
 void hide(Client *c);
-void switch_vdesk(int v);
-#endif /* def VWM */
+void switch_vdesk(ScreenInfo *s, int v);
+#endif
 ScreenInfo *find_screen(Window root);
+ScreenInfo *find_current_screen(void);
+void grab_keys_for_screen(ScreenInfo *s);
 
 /* ewmh.c */
 
