@@ -16,6 +16,7 @@ int ignore_xerror = 0;
 
 /* Now do this by fork()ing twice so we don't have to worry about SIGCHLDs */
 void spawn(const char *const cmd[]) {
+	ScreenInfo *current_screen = find_current_screen();
 	pid_t pid;
 
 	if (current_screen && current_screen->display)
@@ -37,8 +38,8 @@ void spawn(const char *const cmd[]) {
 }
 
 void handle_signal(int signo) {
-	(void)signo;  /* unused */
 	int i;
+	(void)signo;  /* unused */
 	/* SIGCHLD check no longer necessary */
 	/* Quit Nicely */
 	while(head_client) remove_client(head_client);
@@ -46,7 +47,7 @@ void handle_signal(int signo) {
 	if (font) XFreeFont(dpy, font);
 	for (i = 0; i < num_screens; i++) {
 		XFreeGC(dpy, screens[i].invert_gc);
-		XInstallColormap(dpy, DefaultColormap(dpy, screens[i].screen));
+		XInstallColormap(dpy, DefaultColormap(dpy, i));
 	}
 	free(screens);
 	XCloseDisplay(dpy);
@@ -54,8 +55,8 @@ void handle_signal(int signo) {
 }
 
 int handle_xerror(Display *dsply, XErrorEvent *e) {
-	(void)dsply;  /* unused */
 	Client *c;
+	(void)dsply;  /* unused */
 
 	if (ignore_xerror) {
 		LOG_DEBUG("handle_xerror() ignored an XErrorEvent: %d\n", e->error_code);
@@ -83,17 +84,3 @@ int handle_xerror(Display *dsply, XErrorEvent *e) {
 	}
 	return 0;
 }
-
-void grab_keysym(Window w, unsigned int mask, KeySym keysym) {
-	KeyCode keycode = XKeysymToKeycode(dpy, keysym);
-	XGrabKey(dpy, keycode, mask, w, True,
-			GrabModeAsync, GrabModeAsync);
-	XGrabKey(dpy, keycode, mask|LockMask, w, True,
-			GrabModeAsync, GrabModeAsync);
-	if (numlockmask) {
-		XGrabKey(dpy, keycode, mask|numlockmask, w, True,
-				GrabModeAsync, GrabModeAsync);
-		XGrabKey(dpy, keycode, mask|numlockmask|LockMask, w, True,
-				GrabModeAsync, GrabModeAsync);
-	}
-}
