@@ -14,7 +14,6 @@ static void init_geometry(Client *c);
 static void reparent(Client *c);
 static void *get_property(Window w, Atom property, Atom req_type,
 		unsigned long *nitems_return);
-static PropMwmHints *get_mwm_hints(Window);
 #ifdef XDEBUG
 static const char *map_state_string(int map_state);
 static const char *gravity_string(int gravity);
@@ -27,7 +26,6 @@ void make_new_client(Window w, ScreenInfo *s) {
 	Client *c;
 	char *name;
 	XClassHint *class;
-	PropMwmHints *mhints;
 
 	XGrabServer(dpy);
 
@@ -74,14 +72,6 @@ void make_new_client(Window w, ScreenInfo *s) {
 	XUngrabServer(dpy);
 
 	c->border = opt_bw;
-	if ((mhints = get_mwm_hints(c->window))) {
-		if ((mhints->flags & MWM_HINTS_DECORATIONS)
-				&& !(mhints->decorations & MWM_DECOR_ALL)
-				&& !(mhints->decorations & MWM_DECOR_BORDER)) {
-			c->border = 0;
-		}
-		XFree(mhints);
-	}
 
 	init_geometry(c);
 
@@ -151,7 +141,7 @@ void make_new_client(Window w, ScreenInfo *s) {
 	/* Only map the window frame (and thus the window) if it's supposed
 	 * to be visible on this virtual desktop. */
 #ifdef VWM
-	if (vdesk == c->vdesk || is_sticky(c))
+	if (s->vdesk == c->vdesk)
 #endif
 	{
 		unhide(c, RAISE);
@@ -178,14 +168,26 @@ void make_new_client(Window w, ScreenInfo *s) {
 static void init_geometry(Client *c) {
 	long size_flags;
 	XWindowAttributes attr;
+	unsigned long nitems;
+	PropMwmHints *mprop;
 #ifdef VWM
-	unsigned long nitems, i;
+	unsigned long i;
 	unsigned long *lprop;
 	Atom *aprop;
 #endif
 
+	if ( (mprop = get_property(c->window, mwm_hints, mwm_hints, &nitems)) ) {
+		if (nitems >= PROP_MWM_HINTS_ELEMENTS
+				&& (mprop->flags & MWM_HINTS_DECORATIONS)
+				&& !(mprop->decorations & MWM_DECOR_ALL)
+				&& !(mprop->decorations & MWM_DECOR_BORDER)) {
+			c->border = 0;
+		}
+		XFree(mprop);
+	}
+
 #ifdef VWM
-	c->vdesk = vdesk;
+	c->vdesk = c->screen->vdesk;
 	if ( (lprop = get_property(c->window, xa_net_wm_desktop, XA_CARDINAL, &nitems)) ) {
 		if (nitems && valid_vdesk(lprop[0]))
 			c->vdesk = lprop[0];
@@ -253,7 +255,7 @@ static void reparent(Client *c) {
 
 	p_attr.border_pixel = c->screen->bg.pixel;
 	p_attr.override_redirect = True;
-	p_attr.event_mask = ChildMask | ButtonPressMask | ExposureMask | EnterWindowMask;
+	p_attr.event_mask = ChildMask | ButtonPressMask | EnterWindowMask;
 	c->parent = XCreateWindow(dpy, c->screen->root, c->x-c->border, c->y-c->border,
 		c->width, c->height, c->border,
 		DefaultDepth(dpy, c->screen->screen), CopyFromParent,
@@ -264,6 +266,9 @@ static void reparent(Client *c) {
 	XSetWindowBorderWidth(dpy, c->window, 0);
 	XReparentWindow(dpy, c->window, c->parent, 0, 0);
 	XMapWindow(dpy, c->window);
+#ifdef MOUSE
+	grab_button(c->parent, grabmask2, AnyButton);
+#endif
 }
 
 /* Get WM_NORMAL_HINTS property */
@@ -331,18 +336,6 @@ static void *get_property(Window w, Atom property, Atom req_type,
 	return NULL;
 }
 
-static PropMwmHints *get_mwm_hints(Window w) {
-	unsigned long nitems;
-	PropMwmHints *prop;
-	if ( (prop = get_property(w, mwm_hints, mwm_hints, &nitems)) ) {
-		if (nitems >= PROP_MWM_HINTS_ELEMENTS) {
-			return prop;
-		}
-		XFree(prop);
-	}
-	return NULL;
-}
-
 #ifdef XDEBUG
 static const char *map_state_string(int map_state) {
 	const char *map_states[4] = {
