@@ -14,6 +14,7 @@ Window info_window = None;
 static void create_info_window(Client *c);
 static void update_info_window(Client *c);
 static void remove_info_window(void);
+static void grab_keysym(Window w, unsigned int mask, KeySym keysym);
 
 static void create_info_window(Client *c) {
 	info_window = XCreateSimpleWindow(dpy, c->screen->root, -4, -4, 2, 2,
@@ -91,17 +92,7 @@ static void draw_outline(Client *c) {
 }
 #endif
 
-#ifdef MOUSE
-void get_mouse_position(int *x, int *y, Window root) {
-	Window dw1, dw2;
-	int t1, t2;
-	unsigned int t3;
-
-	XQueryPointer(dpy, root, &dw1, &dw2, x, y, &t1, &t2, &t3);
-}
-#endif
-
-void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2) {
+static void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2) {
 	c->width = abs(x1 - x2);
 	c->height = abs(y1 - y2);
 	c->width -= (c->width - c->base_width) % c->width_inc;
@@ -153,6 +144,7 @@ void sweep(Client *c) {
 				XUngrabPointer(dpy, CurrentTime);
 				moveresize(c);
 				return;
+			default: break;
 		}
 	}
 }
@@ -207,8 +199,9 @@ static void snap_client(Client *c) {
 	dx = dy = opt_snap;
 	for (ci = head_client; ci; ci = ci->next) {
 		if (ci != c
+				&& (ci->screen == c->screen)
 #ifdef VWM
-				&& (ci->vdesk == vdesk || is_sticky(ci))
+				&& (ci->vdesk == c->vdesk)
 #endif
 				) {
 			if (ci->y - ci->border - c->border - c->height - c->y <= opt_snap && c->y - c->border - ci->border - ci->height - ci->y <= opt_snap) {
@@ -290,8 +283,7 @@ void drag(Client *c) {
 					moveresize(c);
 				}
 				return;
-			default:
-				break;
+			default: break;
 		}
 	}
 }
@@ -367,7 +359,9 @@ void next(void) {
 			return;
 	}
 #ifdef VWM
-	while (newc->vdesk != vdesk && !is_sticky(newc));
+	/* NOTE: Checking against newc->screen->vdesk implies we can Alt+Tab
+	 * across screen boundaries.  Is this what we want? */
+	while (newc->vdesk != newc->screen->vdesk);
 #else
 	while (0);
 #endif
@@ -382,24 +376,26 @@ void next(void) {
 }
 
 #ifdef VWM
-void switch_vdesk(int v) {
+void switch_vdesk(ScreenInfo *s, int v) {
 	Client *c;
 #ifdef DEBUG
 	int hidden = 0, raised = 0;
 #endif
 
-	if (v == vdesk)
+	if (v == s->vdesk)
 		return;
 	if (current && !is_sticky(current)) {
-		client_update_current(current, NULL);
+		select_client(NULL);
 	}
-	LOG_DEBUG("switch_vdesk(): Switching to desk %d", v);
+	LOG_DEBUG("switch_vdesk(): Switching screen %d to desk %d", s->screen, v);
 	for (c = head_client; c; c = c->next) {
+		if (c->screen != s)
+			continue;
 		if (is_sticky(c) && c->vdesk != v) {
 			c->vdesk = v;
 			update_net_wm_desktop(c);
 		}
-		if (c->vdesk == vdesk) {
+		if (c->vdesk == s->vdesk) {
 			hide(c);
 #ifdef DEBUG
 			hidden++;
@@ -411,7 +407,7 @@ void switch_vdesk(int v) {
 #endif
 		}
 	}
-	vdesk = v;
+	s->vdesk = v;
 	LOG_DEBUG(" (%d hidden, %d raised)\n", hidden, raised);
 }
 #endif /* def VWM */
@@ -424,3 +420,56 @@ ScreenInfo *find_screen(Window root) {
 	}
 	return NULL;
 }
+
+ScreenInfo *find_current_screen(void) {
+	Window cur_root, dw;
+	int di;
+	unsigned int dui;
+
+	/* XQueryPointer is useful for getting the current pointer root */
+	XQueryPointer(dpy, screens[0].root, &cur_root, &dw, &di, &di, &di, &di, &dui);
+	return find_screen(cur_root);
+}
+
+static void grab_keysym(Window w, unsigned int mask, KeySym keysym) {
+	KeyCode keycode = XKeysymToKeycode(dpy, keysym);
+	XGrabKey(dpy, keycode, mask, w, True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey(dpy, keycode, mask|LockMask, w, True,
+			GrabModeAsync, GrabModeAsync);
+	if (numlockmask) {
+		XGrabKey(dpy, keycode, mask|numlockmask, w, True,
+				GrabModeAsync, GrabModeAsync);
+		XGrabKey(dpy, keycode, mask|numlockmask|LockMask, w, True,
+				GrabModeAsync, GrabModeAsync);
+	}
+}
+
+void grab_keys_for_screen(ScreenInfo *s) {
+	KeySym *keysym;
+	KeySym keys_to_grab[] = {
+		KEY_NEW, KEY_KILL,
+		KEY_TOPLEFT, KEY_TOPRIGHT, KEY_BOTTOMLEFT, KEY_BOTTOMRIGHT,
+		KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_UP,
+		KEY_LOWER, KEY_ALTLOWER, KEY_INFO, KEY_MAXVERT, KEY_MAX,
+#ifdef VWM
+		KEY_FIX, KEY_PREVDESK, KEY_NEXTDESK,
+		XK_1, XK_2, XK_3, XK_4, XK_5, XK_6, XK_7, XK_8,
+#endif
+		0
+	};
+	KeySym alt_keys_to_grab[] = {
+		KEY_KILL, KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_UP, 0
+	};
+
+	/* Release any previous grabs */
+	XUngrabKey(dpy, AnyKey, AnyModifier, s->root);
+	/* Grab key combinations we're interested in */
+	for (keysym = keys_to_grab; *keysym; keysym++) {
+		grab_keysym(s->root, grabmask1, *keysym);
+	}
+	for (keysym = alt_keys_to_grab; *keysym; keysym++) {
+		grab_keysym(s->root, grabmask1 | altmask, *keysym);
+	}
+	grab_keysym(s->root, grabmask2, KEY_NEXT);
+}
